package config

import (
	"encoding/json"
	"io/ioutil"
	"log"
	"os"
)

type Config struct {
	User      User      `json:"user"`
	ServerTS  ServerTS  `json:"serverTS"`
	ServerWeb ServerWeb `json:"serverWeb"`
}

type User struct {
	Name     string `json:"name"`
	Password string `json:"password"`
}

type ServerTS struct {
	IP         string `json:"ip"`
	PortServer uint16 `json:"portServer"`
	PortQuery  uint16 `json:"portQuery"`
}

type ServerWeb struct {
	Port uint16 `json:"port"`
}

const FileName = "config.json"

func New() (*Config, error) {
	config := defaults()

	configFile, err := os.Open(FileName)
	if err != nil {
		if err := config.createFile(); err != nil {
			return nil, err
		}

		log.Println(config)
		return &config, nil
	}
	defer configFile.Close()

	jsonParser := json.NewDecoder(configFile)
	jsonParser.Decode(&config)

	return &config, nil
}

func (config Config) createFile() error {
	configJSON, err := json.MarshalIndent(config, "", "    ")
	if err != nil {
		return err
	}

	return ioutil.WriteFile(FileName, configJSON, 0644)
}

func defaults() Config {
	return Config{
		User: User{
			Name:     "serveradmin",
			Password: "",
		},
		ServerTS: ServerTS{
			IP:         "127.0.0.1",
			PortServer: 9987,
			PortQuery:  10011,
		},
		ServerWeb: ServerWeb{
			Port: 80,
		},
	}
}
